# Secure Shell (SSH)

## Clients

on Linux and Mac, all tools are present. on Windows use one of these:

- [Bitvise SSH Client](https://www.bitvise.com/ssh-client-download) (for the SSH tunnel) 
- [MobaXterm](https://mobaxterm.mobatek.net)
- Windows subsystem Linux (WSL), [install](https://learn.microsoft.com/en-us/windows/wsl/install), then install e.g. Ubuntu and install the openssh.
- [VSCode](../Editors/vscode.md)
- Putty, Kitty, ...

## Connect

[How to connect from the Office](../Servers/README.md#how-to-connect-from-the-office) or [How to connect from abroad](../Servers/README.md#how-to-connect-from-home-or-abroad)

**Connect from the office** by typing either of the following in a terminal. Replace `[USERNAME]` with your own.

```bash title="SSH commands"
ssh -X [USERNAME]@srvx1.img.univie.ac.at
ssh -X [USERNAME]@srvx8.img.univie.ac.at
ssh -X [USERNAME]@jet01.img.univie.ac.at
ssh -X [USERNAME]@131.130.157.215
ssh -X [USERNAME]@jet02.img.univie.ac.at
ssh -X [USERNAME]@131.130.157.216
```

The `-X` option enables X11 forwarding via ssh, i.e., permits opening graphical windows. On Windows you need to enter these details to the ssh client.

Consider using a `~/.ssh/config` configuration file to allow easier access like this:

```sh title="./ssh/config"
Host *
    ServerAliveInterval 60
    ServerAliveCountMax 2

Host srvx1
    HostName srvx1.img.univie.ac.at
    User [USERNAME]
Host srvx8
    HostName srvx8.img.univie.ac.at
    User [USERNAME]
Host jet
    HostName jet01.img.univie.ac.at
    User [USERNAME]
Host srvx2jet
    HostName jet01.img.univie.ac.at
    User [USERNAME]
    ProxyJump srvx1.img.univie.ac.at
Host login
    HostName login.univie.ac.at
    User [U:Account USERNAME]
Host ecaccess
    Host ecaccess.ecmwf.int
    HostKeyAlgorithms ssh-dss
    KexAlgorithms diffie-hellman-group1-sha1
    Ciphers aes256-cbc
Host jump.ecmwf.int shell.ecmwf.int
    HostKeyAlgorithms +ssh-rsa*,rsa-sha2-512
    PubkeyAcceptedKeyTypes +ssh-rsa*
    User [ECMWF USERNAME]  
# For ecgate and Cray HPCF
Host ecg* cc*
    HostKeyAlgorithms +ssh-rsa*,rsa-sha2-512
    PubkeyAcceptedKeyTypes +ssh-rsa*
    User [ECMWF USERNAME]
    ProxyJump shell.ecmwf.int
# For Atos HPCF
Host a?-* a??-* hpc-* hpc2020-* ecs-*
    HostKeyAlgorithms +ssh-rsa*,rsa-sha2-512
    PubkeyAcceptedKeyTypes +ssh-rsa*
    User [ECMWF USERNAME]
    ProxyJump jump.ecmwf.int
```
and replacing `[USERNAME]` and `[u:account USERNAME]` with your usernames. Using such a file allows to connect like this `ssh srvx1` using the correct server adress and specified username. Copy this file as well on `login.univie.ac.at` and you can use commands like this: `ssh -t login ssh jet` to connect directly to `jet` via the `login` gateway.

Please note the special algorithms for ecaccess and of course ECMWF uses [teleport](../ECMWF.md#connecting-to-ecmwf-services) now.

**From eduroam**: You should be able to log in as above.

**From the outer world**: use the [VPN](VPN.md) or `srvx1.img.univie.ac.at` as jump host.

If you are a guest, you can apply for a [guest u:account](https://zid.univie.ac.at/en/uaccount/#c14154). This will give you access to eduroam and to the VPN. Your application needs to be endorsed by a staff member, who also determines the expiration date of the account. **Please ask the sponsor first!**

## SSH Authentication with keys

If you want to use ssh-keys you can also use different keys in `.ssh/config` per server with `IdentityFile ~/.ssh/id_rsa_for_server`.

**Note: If you are having trouble with your ssh-key being accepted. You need to check the selinux security policy. Because we use non standard home directories, which are not accepted by default.** Do that by running:
```bash
ls -ldZ ~/.ssh/
drwx------. 2 monkey users system_u:object_r:ssh_home_t:s0 4.0K Aug 18 21:24 .ssh/

ls -lZ ~/.ssh/
-rw-------. 1 monkey users unconfined_u:object_r:ssh_home_t:s0  1.3K Aug 17 10:58 authorized_keys
```
If you are not seeing `ssh_home_t` than you need to run `restorecon -Rv ~/.ssh/` which should update the security policy and allow for the SSH-Daemon to accept your keys.

**Please add your ssh-keys via [IPA](IPA.md#add-ssh-key)**

Find a solution [Questions - How to use ssh-key authentication?](Questions.md#q-how-to-use-ssh-key-authentication) or [Questions - How to use an ssh-agent?](Questions.md#q-how-to-use-an-ssh-agent)

## Connect Script
If you are using a terminal (Mac, Linux, WSL, ...) you can use the script [Download: connect2jet](./connect2jet) like this:
```bash
connect2jet -g [U:Account-Username]@login.univie.ac.at [Jet-Username]@jet01.img.univie.ac.at
```

??? note "connect2jet"

    ```bash title="Connect to Jet"
    --8<-- "SSH-VPN-VNC/connect2jet"
    ```

There is also an option to forward a port, e.g. the VNC Port:

```bash
connect2jet -g [U:Account-Username]@login.univie.ac.at -p 5901 [Jet-Username]@jet01.img.univie.ac.at
```
which allows you to connect to `localhost:5901` and view the VNC session. Other gateway servers can be `srvx1.img.univie.ac.at`

## Tunneling

If you are connected to eduroam or you are on an external computer, you'll need to use an SSH tunnel. The instructions below refer to jet01, but you can do just the same with jet02.

On Linux, start [Remmina](https://remmina.org/), then:

*   Set "Server" to `jet01.img.univie.ac.at:[DISPLAY]` in the "Basic" tab
*   Move to the "SSH Tunnel" tab, checkout "Enable SSH Tunnel", "Same server at port 22" and specify your favourite SSH authentication method.
*   Save and connect.

On Windows, you can use [Bitvise SSH Client](https://www.bitvise.com/ssh-client-download) (for the SSH tunnel) and the [RealVNC VNC Viewer](https://www.realvnc.com/en/connect/download/viewer/windows/) or [MobaXterm](https://mobaxterm.mobatek.net).

Setup might be bit different for different clients, but all need these informationÖ

Option Bitvise SSH Client/MobaXterm and RealVNC:

*   Start the SSH client
*   Go to tab "C2S" or SSH tunnels (port forwarding)
*   Set "Listen Interface" to `127.0.0.1`
*   Set "Listening Port" to `5900+[DISPLAY]`, e.g., `5905`
*   Set "Destination Host" to `jet01.img.univie.ac.at`
*   Set "Destination Port" to `5900+[DISPLAY]`
*   Now start VncViewer and connect to `127.0.0.1:5900+[DISPLAY]`

## SSHFS
It is possible to mount your home directory to your personal computer on Linux via `sshfs` or using of course a dedicated remote file browser like: Filezilla, Cyberduck, ...

on Linux you need to install `fuse2` and `sshfs`, the names might vary between distributions, but are all in the default repos.
```bash
# connect to srvx1 using your home directory and a srvx1 directory on your local computer
# mountserver [host] [remotedir] [localdir]
mkdir -p $HOME/srvx1
mountserver [USER]@srvx1.img.univie.ac.at /users/staff/[USER] $HOME/srvx1
```
Note the directories might vary, depending on your membership (staff, external, students).

## MOSH
Remote terminal application that allows roaming, supports intermittent connectivity, and provides intelligent local echo and line editing of user keystrokes.

Mosh is a replacement for interactive SSH terminals. It's more robust and responsive, especially over Wi-Fi, cellular, and long-distance links.

In order to use:
- install on your client, [instructions](https://mosh.org/#getting)

```bash
$ mosh user@srvx1.img.univie.ac.at
```

## SSH Banner
If you login in to any of the IMGW Servers you will be greeted by a banner showing some information about the system. However, after some time you might not need that information anymore.

run the following and you will never see the banner again.

```bash
touch $HOME/.hushlogin
```
remove that file and you shall see it again.
